import urlparse
import logging
import hashlib

import error
import CurlCommand

# Constants to use:
SUCCESS = 'Sample has been successfully imported'
NO_SPACE = 'No space left on device'


class ValidationError(error.PartekError):
    def __init__(self, value='Error validating user'):
        self.value = value


class TransferError(error.PartekError):
    def __init__(self, value='Error transferring file'):
        self.value = value


class PartekSender:
    """Handles sending of data to Partek Flow.

    """

    user = ""
    __password = ""
    server = ""
    encrypt = "0"
    SALT = "That rug *really* tied the room together, dude. :) "

    log = logging.getLogger('PartekSender')

    def __init__(self, server="", user="", password=""):
        self.server = self.__get_clean_server_name(server)
        self.user = user
        self.encrypt, self.__password = self.__setup_encryption_status(password)
        self._validate_user()

    @staticmethod
    def __get_clean_server_name(server):
        if server.lower().startswith('http'):
            parts = urlparse.urlparse(server)
        else:
            parts = urlparse.urlparse('http://' + server)
        return urlparse.urlunparse([parts.scheme, parts.netloc, '', '', '', ''])

    def _validate_user(self):
        command = CurlCommand.CurlCommand(self.server + '/flow/validate',
                                          {'username': self.user, 'password': self.__password, 'encrypt': self.encrypt})
        self.log.info("Validating %s against %s", self.user, self.server)
        received = command.execute()
        expected = "Valid username and password"
        if expected != received:
            self.log.error("Validation failed: %s", received)
            raise ValidationError(received)
        else:
            self.log.info("Validation succeeded: %s", expected)

    def __setup_encryption_status(self, plaintext):
        command = CurlCommand.CurlCommand(self.server + "/flow/validate?RESTFUL_VERSION")
        out = command.execute(timeout='10')
        if out == "" or out == "OK":
            self.log.warning("Partek Flow server does not support password encryption")
            return "0", plaintext
        else:
            self.log.info("Partek Flow server supports password encryption")
            return "1", self.__get_encrypted_password(plaintext)

    def __get_encrypted_password(self, plaintext):
        return hashlib.sha1(self.SALT + plaintext).hexdigest()

    def send_samples_to_flow(self, bam_list, project):
        sent_files = 0
        for bam in bam_list:
            self.log.info("Sending: %s", bam)
            command = CurlCommand.CurlCommand(self.server + "/flow/import", {
                'username': self.user,
                'password': self.__password,
                'encrypt': self.encrypt,
                'file': '@' + bam,
                'project': project})
            result = command.execute()
            self.log.info('Result: %s', result)
            if SUCCESS in result:
                sent_files += 1
            elif NO_SPACE:
                raise TransferError('Error transferring file, no space left on Flow server', bam)
            else:
                self.log.warning("Error occurred while sending %s to Flow", bam)
        self.log.info("Sent %d files", sent_files)

    def send_reference_to_flow(self, reference, project):
        """Sends a fasta to Partek Flow as a library reference.

        """
        self.log.info("Sending reference fasta: %s", reference)
        command = CurlCommand.CurlCommand(self.server + '/flow/import_library', {
            'username': self.user,
            'password': self.__password,
            'encrypt': self.encrypt,
            'project': project,
            'file_type': 'reference_genome',
            'file': '@' + reference
        }
        )
        result = command.execute()
        self.log.info("Result: %s", result)
