import logging
import os
import json
import shutil

from django.template import loader, Context
from django.conf import settings

# Partek imports
import partek.PartekSender
import partek.barcode_filter
import partek.plugin.Configuration


class Plugin(object):
    log = logging.getLogger('partek.Plugin')

    def __init__(self):
        self._conf = None

    @property
    def configuration(self):
        return self._conf

    @configuration.setter
    def configuration(self, value):
        self._conf = value

    def _find_read_files(self):
        with open(os.path.join(self._conf.basecaller_dir, 'datasets_basecaller.json'), 'r') as f:
            datasets_basecaller = json.load(f)
        bam_list = []
        for dataset in datasets_basecaller['datasets']:
            # Look for unaligned bam in each data set, if non-existent, look for aligned bam.
            unaligned_bam = os.path.join(self._conf.basecaller_dir, dataset['basecaller_bam'])
            aligned_bam = os.path.join(self._conf.alignment_dir, dataset['file_prefix'] + '.bam')
            if os.path.exists(unaligned_bam):
                self.log.info('BAM found: %s', unaligned_bam)
                bam_list.append(unaligned_bam)
            elif os.path.exists(aligned_bam):
                self.log.info('BAM found: %s', aligned_bam)
                bam_list.append(aligned_bam)
            else:
                self.log.warning('BAM not found for dataset: %s', dataset['dataset_name'])
        return bam_list

    def add_run_name(self, bams):
        bam_list = []
        os.mkdir(os.path.join(self._conf.result_dir, 'bam_links'))
        for bam in bams:
            bam_base = os.path.basename(bam)
            link_name = bam_base + '.' + self._conf.run_name + '.bam'
            os.symlink(bam, os.path.join(self._conf.result_dir, 'bam_links', link_name))
            bam_list.append(os.path.join(self._conf.result_dir, 'bam_links', link_name))
        return bam_list

    def launch(self, data=None):
        # Set up Django settings to search for the templates
        settings.configure(TEMPLATE_DIRS=(os.path.join(self._conf.plugin_dir, 'resources', 'templates'),))

        # Clean up data generated by previous runs
        self._clean_previous_runs()

        # Find available reads to analyze/send
        bam_list = self._find_read_files()
        if os.path.exists(self._conf.barcode_filter):
            bam_list = partek.barcode_filter.filter_files(bam_list, partek.barcode_filter.load_filter_from_file(
                self._conf.barcode_filter))
        elif os.path.exists(os.path.join(self._conf.basecaller_dir, 'datasets_basecaller.json')):
            bam_list = partek.barcode_filter.filter_files(
                bam_list,
                partek.barcode_filter.load_filter_from_json(os.path.join(self._conf.basecaller_dir,
                                                                         'datasets_basecaller.json')))
        bam_list = self.add_run_name(bam_list)

        # Send the files to Flow
        status = self._send_to_flow(bam_list)

        # Let the api know that we ran successfully
        return status

    def _send_to_flow(self, bam_list=[]):
        # Only send to Flow if they specified a destination
        try:
            if self._conf.flow_server:
                sender = partek.PartekSender.PartekSender(self._conf.flow_server,
                                                          self._conf.flow_username,
                                                          self._conf.flow_password)
                # Only try to send reference if it exists
                if self._conf.reference_path:
                    sender.send_reference_to_flow(self._conf.reference_path,
                                                  self._conf.flow_project)
                sender.send_samples_to_flow(bam_list, self._conf.flow_project)
        except Exception as e:
            self.log.exception('Exception encountered while sending files to Flow')
            self._write_block_html(len(bam_list), True, str(e))
            return False
        # Write the summary of files uploaded
        self._write_block_html(len(bam_list), False)
        return True

    def _write_block_html(self, files_found, error_occurred, error=''):
        html_context = Context(
            {'dest_server': self._conf.flow_server,
             'project': self._conf.flow_project,
             'files_found': files_found,
             'error_occurred': error_occurred, 'error': error})
        html_template = loader.get_template('partekflow_block_template.html')
        with open(os.path.join(self._conf.result_dir, 'PartekFlowUploader_block.html'), 'w') as f:
            f.write(html_template.render(html_context))

    def _clean_previous_runs(self):
        self.log.debug("Cleaning previous runs")
        extensions = [".png", ".html", ".bam"]
        filenames = ''
        try:
            filenames = os.listdir(self._conf.result_dir)
        except OSError:
            self.log.exception('Unable to list contents of result directory')
        for filename in filenames:
            for ext in extensions:
                if filename.endswith(ext):
                    try:
                        self.log.info('Removing: %s', os.path.join(filename))
                        os.remove(os.path.join(self._conf.result_dir, filename))
                    except OSError:
                        self.log.exception('Unable to remove: %s', filename)
        shutil.rmtree(os.path.join(self._conf.result_dir, 'bam_links'),
                      onerror=lambda func, arg, exc: self.log.warning("Error removing link: %s", arg))
