import unittest
import os
import shutil
from django.conf import settings
import partek.plugin.Configuration


class TestPlugin(unittest.TestCase):
    test_plugin = None
    test_bams = None
    template_dir = os.path.abspath(os.path.join('resources', 'templates'))
    TEST_OUTPUT_DIR = 'TestPlugin_output_dir'
    print "Setting template dir: %s" % template_dir
    settings.configure(TEMPLATE_DIRS=(template_dir,))

    def setUp(self):
        test_configuration = partek.plugin.Configuration()
        test_configuration.result_dir = os.path.join(self.TEST_OUTPUT_DIR, 'temp_result_dir')
        test_configuration.basecaller_dir = os.path.join(self.TEST_OUTPUT_DIR, 'temp_basecaller_dir')
        test_configuration.flow_project = 'test_project'
        test_configuration.flow_server = 'test_server'
        test_configuration.run_name = 'test_run'

        self.test_plugin = partek.plugin.Plugin()
        self.test_plugin.configuration = test_configuration

        os.mkdir(self.TEST_OUTPUT_DIR)
        os.mkdir(self.test_plugin.configuration.result_dir)
        os.mkdir(self.test_plugin.configuration.basecaller_dir)
        self._create_test_bams()

    def tearDown(self):
        shutil.rmtree(self.TEST_OUTPUT_DIR)

    def _create_test_bams(self):
        bams = ['test1.bam', 'test2.bam', 'test3.bam']
        self.test_bams = []
        for bam in bams:
            self.test_bams.append(os.path.abspath(os.path.join(self.TEST_OUTPUT_DIR, 'temp_result_dir', bam)))
        for bam in self.test_bams:
            with open(os.path.join(self.test_plugin.configuration.result_dir, bam), 'w') as fd:
                fd.write(bam + '\n')

    def test_add_run_name(self):
        self.assertFalse(os.path.exists(os.path.join(self.test_plugin.configuration.result_dir, 'bam_links')))
        observed_bams = self.test_plugin.add_run_name(self.test_bams)
        expected_bams = [
            os.path.join(self.TEST_OUTPUT_DIR, 'temp_result_dir', 'bam_links', 'test1.bam.test_run.bam'),
            os.path.join(self.TEST_OUTPUT_DIR, 'temp_result_dir', 'bam_links', 'test2.bam.test_run.bam'),
            os.path.join(self.TEST_OUTPUT_DIR, 'temp_result_dir', 'bam_links', 'test3.bam.test_run.bam')
        ]
        self.assertEqual(expected_bams, observed_bams)
        for bam in expected_bams:
            self.assertTrue(os.path.exists(bam), 'BAM file does not exist: %s' % bam)
            self.assertTrue(os.path.islink(bam))

    def test_clean_previous_runs(self):
        fake_test_files = ['clean_test.png', 'clean_test.html', 'clean_test.bam']
        os.mkdir(os.path.join(self.test_plugin.configuration.result_dir, 'bam_links'))
        for fake_file in fake_test_files:
            with open(os.path.join(self.test_plugin.configuration.result_dir, fake_file), 'w') as fd:
                fd.write("fake_test\n")
            self.assertTrue(os.path.exists(os.path.join(self.test_plugin.configuration.result_dir, fake_file)))
        self.test_plugin._clean_previous_runs()

        for fake_file in fake_test_files:
            self.assertFalse(os.path.exists(os.path.join(self.test_plugin.configuration.result_dir, fake_file)))

    def test_write_block_html(self):
        block_path = os.path.join(self.test_plugin.configuration.result_dir, 'PartekFlowUploader_block.html')
        self.assertFalse(os.path.exists(block_path))
        self.test_plugin._write_block_html(10, False)
        self.assertTrue(os.path.exists(block_path), 'Failed to write block html to plugin result directory')
        with open(block_path, 'r') as fd:
            observed = fd.read()
        expected = '<html>' \
                   '\n\t<body>' \
                   '\n\t\t\n\t\t\t\n\t\t\t' \
                   'The samples are now available in <a href="test_server/flow" target="_blank">Partek Flow</a>' \
                   ' for further analysis.<br />' \
                   '\n\t\t\tThe samples have been added to the <b>test_project</b> project.' \
                   '<br />\n\t\t\t\n\t\t\n\t</body>\n</html>\n\n'

        self.assertEqual(expected, observed)

    def test_write_block_html_one_file(self):
        block_path = os.path.join(self.test_plugin.configuration.result_dir, 'PartekFlowUploader_block.html')
        self.assertFalse(os.path.exists(block_path))
        self.test_plugin._write_block_html(1, False)
        self.assertTrue(os.path.exists(block_path), 'Failed to write block html to plugin result directory')
        with open(block_path, 'r') as fd:
            observed = fd.read()
        expected = '<html>' \
                   '\n\t<body>' \
                   '\n\t\t\n\t\t\t\n\t\t\t' \
                   'The sample is now available in <a href="test_server/flow" target="_blank">Partek Flow</a>' \
                   ' for further analysis.<br />' \
                   '\n\t\t\tThe sample has been added to the <b>test_project</b> project.' \
                   '<br />\n\t\t\t\n\t\t\n\t</body>\n</html>\n\n'

        self.assertEqual(expected, observed)

    def test_write_block_html_error(self):
        block_path = os.path.join(self.test_plugin.configuration.result_dir, 'PartekFlowUploader_block.html')
        self.assertFalse(os.path.exists(block_path))
        self.test_plugin._write_block_html(12, True, 'Connection timed out')
        self.assertTrue(os.path.exists(block_path), 'Failed to write block html')
        with open(block_path, 'r') as fd:
            observed = fd.read()
        expected = '<html>' \
                   '\n\t<body>' \
                   '\n\t\t\n\t\t' \
                   'An error occurred while sending files to Flow. ' \
                   'Check the log for more details. <br />' \
                   '\n\t\t' \
                   'Error: Connection timed out <br />' \
                   '\n\t\t\n\t</body>\n</html>\n\n'

        self.assertEqual(expected, observed)
